# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/admin/participant_pruning_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n>com/daml/ledger/api/v1/admin/participant_pruning_service.proto\x12\x1c\x63om.daml.ledger.api.v1.admin\"\x94\x01\n\x0cPruneRequest\x12\x1e\n\x0bprune_up_to\x18\x01 \x01(\tR\tpruneUpTo\x12#\n\rsubmission_id\x18\x02 \x01(\tR\x0csubmissionId\x12?\n\x1cprune_all_divulged_contracts\x18\x03 \x01(\x08R\x19pruneAllDivulgedContracts\"\x0f\n\rPruneResponse2}\n\x19ParticipantPruningService\x12`\n\x05Prune\x12*.com.daml.ledger.api.v1.admin.PruneRequest\x1a+.com.daml.ledger.api.v1.admin.PruneResponseB\xaf\x01\n\x1c\x63om.daml.ledger.api.v1.adminB#ParticipantPruningServiceOuterClassZKgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1/admin\xaa\x02\x1c\x43om.Daml.Ledger.Api.V1.Adminb\x06proto3')



_PRUNEREQUEST = DESCRIPTOR.message_types_by_name['PruneRequest']
_PRUNERESPONSE = DESCRIPTOR.message_types_by_name['PruneResponse']
PruneRequest = _reflection.GeneratedProtocolMessageType('PruneRequest', (_message.Message,), {
  'DESCRIPTOR' : _PRUNEREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.admin.participant_pruning_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.PruneRequest)
  })
_sym_db.RegisterMessage(PruneRequest)

PruneResponse = _reflection.GeneratedProtocolMessageType('PruneResponse', (_message.Message,), {
  'DESCRIPTOR' : _PRUNERESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.admin.participant_pruning_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.PruneResponse)
  })
_sym_db.RegisterMessage(PruneResponse)

_PARTICIPANTPRUNINGSERVICE = DESCRIPTOR.services_by_name['ParticipantPruningService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\034com.daml.ledger.api.v1.adminB#ParticipantPruningServiceOuterClassZKgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1/admin\252\002\034Com.Daml.Ledger.Api.V1.Admin'
  _PRUNEREQUEST._serialized_start=97
  _PRUNEREQUEST._serialized_end=245
  _PRUNERESPONSE._serialized_start=247
  _PRUNERESPONSE._serialized_end=262
  _PARTICIPANTPRUNINGSERVICE._serialized_start=264
  _PARTICIPANTPRUNINGSERVICE._serialized_end=389
# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
:mod:`dazl.values` package
==========================

The :mod:`dazl.values` module contains utilities for converting between different representations
of DAML-LF types.
"""

from .canonical import CanonicalMapper
from .context import Context
from .json import JsonDecoder, JsonEncoder
from .mapper import ValueMapper
from .protobuf import ProtobufDecoder, ProtobufEncoder
from .string import ArrayStringMapper, FlatStringMapper



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/transaction_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import ledger_offset_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_ledger__offset__pb2
from . import transaction_filter_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_transaction__filter__pb2
from . import transaction_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_transaction__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n0com/daml/ledger/api/v1/transaction_service.proto\x12\x16\x63om.daml.ledger.api.v1\x1a*com/daml/ledger/api/v1/ledger_offset.proto\x1a/com/daml/ledger/api/v1/transaction_filter.proto\x1a(com/daml/ledger/api/v1/transaction.proto\"\x86\x02\n\x16GetTransactionsRequest\x12\x1b\n\tledger_id\x18\x01 \x01(\tR\x08ledgerId\x12:\n\x05\x62\x65gin\x18\x02 \x01(\x0b\x32$.com.daml.ledger.api.v1.LedgerOffsetR\x05\x62\x65gin\x12\x36\n\x03\x65nd\x18\x03 \x01(\x0b\x32$.com.daml.ledger.api.v1.LedgerOffsetR\x03\x65nd\x12\x41\n\x06\x66ilter\x18\x04 \x01(\x0b\x32).com.daml.ledger.api.v1.TransactionFilterR\x06\x66ilter\x12\x18\n\x07verbose\x18\x05 \x01(\x08R\x07verbose\"b\n\x17GetTransactionsResponse\x12G\n\x0ctransactions\x18\x01 \x03(\x0b\x32#.com.daml.ledger.api.v1.TransactionR\x0ctransactions\"j\n\x1bGetTransactionTreesResponse\x12K\n\x0ctransactions\x18\x01 \x03(\x0b\x32\'.com.daml.ledger.api.v1.TransactionTreeR\x0ctransactions\"\x87\x01\n\x1eGetTransactionByEventIdRequest\x12\x1b\n\tledger_id\x18\x01 \x01(\tR\x08ledgerId\x12\x19\n\x08\x65vent_id\x18\x02 \x01(\tR\x07\x65ventId\x12-\n\x12requesting_parties\x18\x03 \x03(\tR\x11requestingParties\"\x8e\x01\n\x19GetTransactionByIdRequest\x12\x1b\n\tledger_id\x18\x01 \x01(\tR\x08ledgerId\x12%\n\x0etransaction_id\x18\x02 \x01(\tR\rtransactionId\x12-\n\x12requesting_parties\x18\x03 \x03(\tR\x11requestingParties\"c\n\x16GetTransactionResponse\x12I\n\x0btransaction\x18\x01 \x01(\x0b\x32\'.com.daml.ledger.api.v1.TransactionTreeR\x0btransaction\"c\n\x1aGetFlatTransactionResponse\x12\x45\n\x0btransaction\x18\x01 \x01(\x0b\x32#.com.daml.ledger.api.v1.TransactionR\x0btransaction\"2\n\x13GetLedgerEndRequest\x12\x1b\n\tledger_id\x18\x01 \x01(\tR\x08ledgerId\"T\n\x14GetLedgerEndResponse\x12<\n\x06offset\x18\x01 \x01(\x0b\x32$.com.daml.ledger.api.v1.LedgerOffsetR\x06offset2\xfd\x06\n\x12TransactionService\x12t\n\x0fGetTransactions\x12..com.daml.ledger.api.v1.GetTransactionsRequest\x1a/.com.daml.ledger.api.v1.GetTransactionsResponse0\x01\x12|\n\x13GetTransactionTrees\x12..com.daml.ledger.api.v1.GetTransactionsRequest\x1a\x33.com.daml.ledger.api.v1.GetTransactionTreesResponse0\x01\x12\x81\x01\n\x17GetTransactionByEventId\x12\x36.com.daml.ledger.api.v1.GetTransactionByEventIdRequest\x1a..com.daml.ledger.api.v1.GetTransactionResponse\x12w\n\x12GetTransactionById\x12\x31.com.daml.ledger.api.v1.GetTransactionByIdRequest\x1a..com.daml.ledger.api.v1.GetTransactionResponse\x12\x89\x01\n\x1bGetFlatTransactionByEventId\x12\x36.com.daml.ledger.api.v1.GetTransactionByEventIdRequest\x1a\x32.com.daml.ledger.api.v1.GetFlatTransactionResponse\x12\x7f\n\x16GetFlatTransactionById\x12\x31.com.daml.ledger.api.v1.GetTransactionByIdRequest\x1a\x32.com.daml.ledger.api.v1.GetFlatTransactionResponse\x12i\n\x0cGetLedgerEnd\x12+.com.daml.ledger.api.v1.GetLedgerEndRequest\x1a,.com.daml.ledger.api.v1.GetLedgerEndResponseB\x96\x01\n\x16\x63om.daml.ledger.api.v1B\x1cTransactionServiceOuterClassZEgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1\xaa\x02\x16\x43om.Daml.Ledger.Api.V1b\x06proto3')



_GETTRANSACTIONSREQUEST = DESCRIPTOR.message_types_by_name['GetTransactionsRequest']
_GETTRANSACTIONSRESPONSE = DESCRIPTOR.message_types_by_name['GetTransactionsResponse']
_GETTRANSACTIONTREESRESPONSE = DESCRIPTOR.message_types_by_name['GetTransactionTreesResponse']
_GETTRANSACTIONBYEVENTIDREQUEST = DESCRIPTOR.message_types_by_name['GetTransactionByEventIdRequest']
_GETTRANSACTIONBYIDREQUEST = DESCRIPTOR.message_types_by_name['GetTransactionByIdRequest']
_GETTRANSACTIONRESPONSE = DESCRIPTOR.message_types_by_name['GetTransactionResponse']
_GETFLATTRANSACTIONRESPONSE = DESCRIPTOR.message_types_by_name['GetFlatTransactionResponse']
_GETLEDGERENDREQUEST = DESCRIPTOR.message_types_by_name['GetLedgerEndRequest']
_GETLEDGERENDRESPONSE = DESCRIPTOR.message_types_by_name['GetLedgerEndResponse']
GetTransactionsRequest = _reflection.GeneratedProtocolMessageType('GetTransactionsRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETTRANSACTIONSREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.transaction_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetTransactionsRequest)
  })
_sym_db.RegisterMessage(GetTransactionsRequest)

GetTransactionsResponse = _reflection.GeneratedProtocolMessageType('GetTransactionsResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETTRANSACTIONSRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.transaction_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetTransactionsResponse)
  })
_sym_db.RegisterMessage(GetTransactionsResponse)

GetTransactionTreesResponse = _reflection.GeneratedProtocolMessageType('GetTransactionTreesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETTRANSACTIONTREESRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.transaction_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetTransactionTreesResponse)
  })
_sym_db.RegisterMessage(GetTransactionTreesResponse)

GetTransactionByEventIdRequest = _reflection.GeneratedProtocolMessageType('GetTransactionByEventIdRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETTRANSACTIONBYEVENTIDREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.transaction_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetTransactionByEventIdRequest)
  })
_sym_db.RegisterMessage(GetTransactionByEventIdRequest)

GetTransactionByIdRequest = _reflection.GeneratedProtocolMessageType('GetTransactionByIdRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETTRANSACTIONBYIDREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.transaction_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetTransactionByIdRequest)
  })
_sym_db.RegisterMessage(GetTransactionByIdRequest)

GetTransactionResponse = _reflection.GeneratedProtocolMessageType('GetTransactionResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETTRANSACTIONRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.transaction_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetTransactionResponse)
  })
_sym_db.RegisterMessage(GetTransactionResponse)

GetFlatTransactionResponse = _reflection.GeneratedProtocolMessageType('GetFlatTransactionResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETFLATTRANSACTIONRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.transaction_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetFlatTransactionResponse)
  })
_sym_db.RegisterMessage(GetFlatTransactionResponse)

GetLedgerEndRequest = _reflection.GeneratedProtocolMessageType('GetLedgerEndRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETLEDGERENDREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.transaction_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetLedgerEndRequest)
  })
_sym_db.RegisterMessage(GetLedgerEndRequest)

GetLedgerEndResponse = _reflection.GeneratedProtocolMessageType('GetLedgerEndResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETLEDGERENDRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.transaction_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetLedgerEndResponse)
  })
_sym_db.RegisterMessage(GetLedgerEndResponse)

_TRANSACTIONSERVICE = DESCRIPTOR.services_by_name['TransactionService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026com.daml.ledger.api.v1B\034TransactionServiceOuterClassZEgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1\252\002\026Com.Daml.Ledger.Api.V1'
  _GETTRANSACTIONSREQUEST._serialized_start=212
  _GETTRANSACTIONSREQUEST._serialized_end=474
  _GETTRANSACTIONSRESPONSE._serialized_start=476
  _GETTRANSACTIONSRESPONSE._serialized_end=574
  _GETTRANSACTIONTREESRESPONSE._serialized_start=576
  _GETTRANSACTIONTREESRESPONSE._serialized_end=682
  _GETTRANSACTIONBYEVENTIDREQUEST._serialized_start=685
  _GETTRANSACTIONBYEVENTIDREQUEST._serialized_end=820
  _GETTRANSACTIONBYIDREQUEST._serialized_start=823
  _GETTRANSACTIONBYIDREQUEST._serialized_end=965
  _GETTRANSACTIONRESPONSE._serialized_start=967
  _GETTRANSACTIONRESPONSE._serialized_end=1066
  _GETFLATTRANSACTIONRESPONSE._serialized_start=1068
  _GETFLATTRANSACTIONRESPONSE._serialized_end=1167
  _GETLEDGERENDREQUEST._serialized_start=1169
  _GETLEDGERENDREQUEST._serialized_end=1219
  _GETLEDGERENDRESPONSE._serialized_start=1221
  _GETLEDGERENDRESPONSE._serialized_end=1305
  _TRANSACTIONSERVICE._serialized_start=1308
  _TRANSACTIONSERVICE._serialized_end=2201
# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/command_submission_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import commands_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_commands__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n7com/daml/ledger/api/v1/command_submission_service.proto\x12\x16\x63om.daml.ledger.api.v1\x1a%com/daml/ledger/api/v1/commands.proto\x1a\x1bgoogle/protobuf/empty.proto\"M\n\rSubmitRequest\x12<\n\x08\x63ommands\x18\x01 \x01(\x0b\x32 .com.daml.ledger.api.v1.CommandsR\x08\x63ommands2c\n\x18\x43ommandSubmissionService\x12G\n\x06Submit\x12%.com.daml.ledger.api.v1.SubmitRequest\x1a\x16.google.protobuf.EmptyB\x9c\x01\n\x16\x63om.daml.ledger.api.v1B\"CommandSubmissionServiceOuterClassZEgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1\xaa\x02\x16\x43om.Daml.Ledger.Api.V1b\x06proto3')



_SUBMITREQUEST = DESCRIPTOR.message_types_by_name['SubmitRequest']
SubmitRequest = _reflection.GeneratedProtocolMessageType('SubmitRequest', (_message.Message,), {
  'DESCRIPTOR' : _SUBMITREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.command_submission_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.SubmitRequest)
  })
_sym_db.RegisterMessage(SubmitRequest)

_COMMANDSUBMISSIONSERVICE = DESCRIPTOR.services_by_name['CommandSubmissionService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026com.daml.ledger.api.v1B\"CommandSubmissionServiceOuterClassZEgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1\252\002\026Com.Daml.Ledger.Api.V1'
  _SUBMITREQUEST._serialized_start=151
  _SUBMITREQUEST._serialized_end=228
  _COMMANDSUBMISSIONSERVICE._serialized_start=230
  _COMMANDSUBMISSIONSERVICE._serialized_end=329
# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

from typing import Awaitable, Callable, TypeVar, Union

from ..protocols.events import BaseEvent
from .commands import EventHandlerResponse

E = TypeVar("E", bound=BaseEvent)
T = TypeVar("T")


EventHandler = Callable[[E], EventHandlerResponse]
AEventHandler = Callable[[E], Union[EventHandlerResponse, Awaitable[EventHandlerResponse]]]


EventHandlerDecorator = Callable[[EventHandler[E]], EventHandler[E]]
AEventHandlerDecorator = Callable[[AEventHandler[E]], AEventHandler[E]]


def fluentize(fn: "Callable[[T], None]", *args, **kwargs) -> "Callable[[T], T]":
    """
    Convert a function that takes a value and returns ``None`` to a function that takes a value
    and returns that value. Additional positional and keyword arguments are passed to the function
    unchanged.
    """
    # Note that there isn't a perfect way to expose the true type signature of this method, which
    # is that it matches the input arguments from the original function and merely changes its
    # return value. All sites where this is used should take care to
    from functools import wraps

    @wraps(fn)
    def impl(*i_args, **i_kwargs):
        fn(*args, *i_args, **kwargs, **i_kwargs)
        return args[0] if args else None

    return impl



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

from asyncio import ensure_future, gather, get_event_loop, sleep, wait_for
from concurrent.futures import ThreadPoolExecutor
from datetime import timedelta
from typing import Awaitable, Callable, Dict, Optional, TypeVar

from . import PackageService
from ... import LOG
from ...damlast.daml_lf_1 import Package, PackageRef
from ...damlast.errors import PackageNotFoundError
from ...damlast.lookup import STAR, MultiPackageLookup, PackageExceptionTracker
from ...damlast.parse import parse_archive
from ...damlast.pkgfile import Dar
from ...prim import DazlError

__all__ = ["PackageLoader", "DEFAULT_TIMEOUT"]

T = TypeVar("T")

# mypy insists on having a type annotation here, or it will complain about not being able to
# determine the type of this field in pkgloader_aio_compat.py
DEFAULT_TIMEOUT: timedelta = timedelta(seconds=30)


class PackageLoader:
    """
    Loader for packages from a remote PackageService.

    This class handles retries and backoffs, and avoids having more than one request in flight for
    the same package ID. It is intended to be shared by all local clients that may need package
    information.
    """

    _allow_deprecated_identifiers = False

    def __init__(
        self,
        package_lookup: "MultiPackageLookup",
        conn: "Optional[PackageService]" = None,
        timeout: "Optional[timedelta]" = DEFAULT_TIMEOUT,
        executor: "Optional[ThreadPoolExecutor]" = None,
    ):
        self._package_lookup = package_lookup
        self._conn = conn
        self._timeout = timeout or DEFAULT_TIMEOUT
        self._futures = dict()  # type: Dict[PackageRef, Awaitable[Package]]
        self._executor = executor or ThreadPoolExecutor(3)

    def set_connection(self, conn: "Optional[PackageService]"):
        self._conn = conn

    async def do_with_retry(self, fn: "Callable[[], T]") -> "T":
        """
        Perform a synchronous action that assumes the existence of one or more packages. In the
        event the function raises :class:`PackageNotFoundError` or a wildcarded
        :class:`NameNotFoundError`, the required package/type is fetched and the operation retried.

        If, after a retry, an expected package or type could not be found, the exception is
        re-raised to the caller.

        :param fn: A function to invoke.
        :return: The result of that function.
        """
        guard = PackageExceptionTracker(
            allow_deprecated_identifiers=self._allow_deprecated_identifiers
        )
        while True:
            pkg_ref = guard.pop_package()
            while pkg_ref is not None:
                await self.load(pkg_ref)
                pkg_ref = guard.pop_package()

            with guard:
                return fn()

    async def preload(self, *contents: "Dar") -> None:
        """
        Populate a :class:`PackageCache` with types from DARs.

        :param contents:
            One or more DARs to load into a local package cache.
        """

    async def load(self, ref: "PackageRef") -> "Optional[Package]":
        """
        Load a package ID from the remote server. If the package has additional dependencies, they
        are also loaded.

        :param ref: One or more :class:`PackageRef`s.
        :raises: PackageNotFoundError if the package could not be resolved
        """
        if ref == STAR:
            await self.load_all()
            return None

        # If the package has already been loaded, then skip all the expensive I/O stuff
        try:
            return self._package_lookup.package(ref)
        except PackageNotFoundError:
            pass

        # If we already have a request in-flight, simply return that same Future to our caller;
        # do not try to schedule a new request
        fut = self._futures.get(ref)
        if fut is None:
            fut = ensure_future(self._load(ref))
            self._futures[ref] = fut
        package = await fut

        _ = self._futures.pop(ref, None)

        return package

    async def _load(self, package_id: "PackageRef") -> "Package":
        LOG.info("Loading package: %s", package_id)

        loop = get_event_loop()
        conn = self._conn
        if conn is None:
            raise DazlError("a connection is not configured")

        archive_bytes = await wait_for(
            self.__fetch_package_bytes(conn, package_id), timeout=self._timeout.total_seconds()
        )

        LOG.info("Loaded for package: %s, %d bytes", package_id, len(archive_bytes))

        archive = await loop.run_in_executor(
            self._executor, lambda: parse_archive(package_id, archive_bytes)
        )
        self._package_lookup.add_archive(archive)
        return archive.package

    @staticmethod
    async def __fetch_package_bytes(conn: "PackageService", package_id: "PackageRef") -> bytes:
        sleep_interval = 1

        while True:
            # noinspection PyBroadException
            try:
                return await conn.get_package(package_id)
            except Exception:
                # We tried fetching the package but got an error. Retry, backing off to waiting as
                # much as 30 seconds between each attempt.
                await sleep(sleep_interval)
                sleep_interval = min(sleep_interval * 2, 30)
                LOG.exception("Failed to fetch package; this will be retried.")

    async def load_all(self) -> None:
        """
        Load all packages from the remote server.
        """
        conn = self._conn
        if conn is None:
            raise DazlError("a connection is not configured")

        package_ids = set(await conn.list_package_ids())
        package_ids -= self._package_lookup.package_ids()
        if package_ids:
            await gather(*(self.load(package_id) for package_id in package_ids))

        return None



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/ledger_configuration_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n9com/daml/ledger/api/v1/ledger_configuration_service.proto\x12\x16\x63om.daml.ledger.api.v1\x1a\x1egoogle/protobuf/duration.proto\"<\n\x1dGetLedgerConfigurationRequest\x12\x1b\n\tledger_id\x18\x01 \x01(\tR\x08ledgerId\"\x80\x01\n\x1eGetLedgerConfigurationResponse\x12^\n\x14ledger_configuration\x18\x01 \x01(\x0b\x32+.com.daml.ledger.api.v1.LedgerConfigurationR\x13ledgerConfiguration\"z\n\x13LedgerConfiguration\x12W\n\x1amax_deduplication_duration\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationR\x18maxDeduplicationDurationJ\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03\x32\xa8\x01\n\x1aLedgerConfigurationService\x12\x89\x01\n\x16GetLedgerConfiguration\x12\x35.com.daml.ledger.api.v1.GetLedgerConfigurationRequest\x1a\x36.com.daml.ledger.api.v1.GetLedgerConfigurationResponse0\x01\x42\x9e\x01\n\x16\x63om.daml.ledger.api.v1B$LedgerConfigurationServiceOuterClassZEgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1\xaa\x02\x16\x43om.Daml.Ledger.Api.V1b\x06proto3')



_GETLEDGERCONFIGURATIONREQUEST = DESCRIPTOR.message_types_by_name['GetLedgerConfigurationRequest']
_GETLEDGERCONFIGURATIONRESPONSE = DESCRIPTOR.message_types_by_name['GetLedgerConfigurationResponse']
_LEDGERCONFIGURATION = DESCRIPTOR.message_types_by_name['LedgerConfiguration']
GetLedgerConfigurationRequest = _reflection.GeneratedProtocolMessageType('GetLedgerConfigurationRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETLEDGERCONFIGURATIONREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.ledger_configuration_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetLedgerConfigurationRequest)
  })
_sym_db.RegisterMessage(GetLedgerConfigurationRequest)

GetLedgerConfigurationResponse = _reflection.GeneratedProtocolMessageType('GetLedgerConfigurationResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETLEDGERCONFIGURATIONRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.ledger_configuration_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetLedgerConfigurationResponse)
  })
_sym_db.RegisterMessage(GetLedgerConfigurationResponse)

LedgerConfiguration = _reflection.GeneratedProtocolMessageType('LedgerConfiguration', (_message.Message,), {
  'DESCRIPTOR' : _LEDGERCONFIGURATION,
  '__module__' : 'com.daml.ledger.api.v1.ledger_configuration_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.LedgerConfiguration)
  })
_sym_db.RegisterMessage(LedgerConfiguration)

_LEDGERCONFIGURATIONSERVICE = DESCRIPTOR.services_by_name['LedgerConfigurationService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026com.daml.ledger.api.v1B$LedgerConfigurationServiceOuterClassZEgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1\252\002\026Com.Daml.Ledger.Api.V1'
  _GETLEDGERCONFIGURATIONREQUEST._serialized_start=117
  _GETLEDGERCONFIGURATIONREQUEST._serialized_end=177
  _GETLEDGERCONFIGURATIONRESPONSE._serialized_start=180
  _GETLEDGERCONFIGURATIONRESPONSE._serialized_end=308
  _LEDGERCONFIGURATION._serialized_start=310
  _LEDGERCONFIGURATION._serialized_end=432
  _LEDGERCONFIGURATIONSERVICE._serialized_start=435
  _LEDGERCONFIGURATIONSERVICE._serialized_end=603
# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
"""
Default ``repr`` rules for dazl.

THIS IS NOT A PUBLIC API, AND ITS CONTENTS ARE SUBJECT TO CHANGE AT ANY TIME!
"""

import builtins
from reprlib import Repr
from typing import Collection

REPR = Repr()

# Python's default max string length is a little too short to nicely display some common
# Ledger API constructs such as template IDs and party literals, so bump it up higher
REPR.maxstring = 255


__all__ = ["list", "str", "REPR"]


# noinspection PyShadowingBuiltins
def list(__obj: Collection) -> builtins.str:
    # the typing rules for reprlib are too strict; in reality `repr_list` takes any
    # iterable
    return REPR.repr_list(__obj, REPR.maxlevel)  # type: ignore


# noinspection PyShadowingBuiltins
def str(__obj: builtins.str) -> builtins.str:
    return REPR.repr_str(__obj, REPR.maxlevel)



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/daml_lf_1_14/daml_lf.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import daml_lf_1_pb2 as com_dot_daml_dot_daml__lf__1__14_dot_daml__lf__1__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n#com/daml/daml_lf_1_14/daml_lf.proto\x12\tdaml_lf_1\x1a%com/daml/daml_lf_1_14/daml_lf_1.proto\"m\n\x0e\x41rchivePayload\x12\x14\n\x05minor\x18\x03 \x01(\tR\x05minor\x12\x30\n\tdaml_lf_1\x18\x02 \x01(\x0b\x32\x12.daml_lf_1.PackageH\x00R\x07\x64\x61mlLf1B\x05\n\x03SumJ\x06\x08\x8fN\x10\x90NJ\x04\x08\x01\x10\x02\"u\n\x07\x41rchive\x12<\n\rhash_function\x18\x01 \x01(\x0e\x32\x17.daml_lf_1.HashFunctionR\x0chashFunction\x12\x18\n\x07payload\x18\x03 \x01(\x0cR\x07payload\x12\x12\n\x04hash\x18\x04 \x01(\tR\x04hash*\x1a\n\x0cHashFunction\x12\n\n\x06SHA256\x10\x00\x42|\n\x15\x63om.daml.daml_lf_1_14ZDgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/daml_lf_1_14\xaa\x02\x1c\x43om.Daml.Daml_Lf_1_14.DamlLfb\x06proto3')

_HASHFUNCTION = DESCRIPTOR.enum_types_by_name['HashFunction']
HashFunction = enum_type_wrapper.EnumTypeWrapper(_HASHFUNCTION)
SHA256 = 0


_ARCHIVEPAYLOAD = DESCRIPTOR.message_types_by_name['ArchivePayload']
_ARCHIVE = DESCRIPTOR.message_types_by_name['Archive']
ArchivePayload = _reflection.GeneratedProtocolMessageType('ArchivePayload', (_message.Message,), {
  'DESCRIPTOR' : _ARCHIVEPAYLOAD,
  '__module__' : 'com.daml.daml_lf_1_14.daml_lf_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.ArchivePayload)
  })
_sym_db.RegisterMessage(ArchivePayload)

Archive = _reflection.GeneratedProtocolMessageType('Archive', (_message.Message,), {
  'DESCRIPTOR' : _ARCHIVE,
  '__module__' : 'com.daml.daml_lf_1_14.daml_lf_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.Archive)
  })
_sym_db.RegisterMessage(Archive)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\025com.daml.daml_lf_1_14ZDgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/daml_lf_1_14\252\002\034Com.Daml.Daml_Lf_1_14.DamlLf'
  _HASHFUNCTION._serialized_start=319
  _HASHFUNCTION._serialized_end=345
  _ARCHIVEPAYLOAD._serialized_start=89
  _ARCHIVEPAYLOAD._serialized_end=198
  _ARCHIVE._serialized_start=200
  _ARCHIVE._serialized_end=317
# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/command_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import commands_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_commands__pb2
from . import transaction_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_transaction__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n,com/daml/ledger/api/v1/command_service.proto\x12\x16\x63om.daml.ledger.api.v1\x1a%com/daml/ledger/api/v1/commands.proto\x1a(com/daml/ledger/api/v1/transaction.proto\x1a\x1bgoogle/protobuf/empty.proto\"T\n\x14SubmitAndWaitRequest\x12<\n\x08\x63ommands\x18\x01 \x01(\x0b\x32 .com.daml.ledger.api.v1.CommandsR\x08\x63ommands\"{\n%SubmitAndWaitForTransactionIdResponse\x12%\n\x0etransaction_id\x18\x01 \x01(\tR\rtransactionId\x12+\n\x11\x63ompletion_offset\x18\x02 \x01(\tR\x10\x63ompletionOffset\"\x99\x01\n#SubmitAndWaitForTransactionResponse\x12\x45\n\x0btransaction\x18\x01 \x01(\x0b\x32#.com.daml.ledger.api.v1.TransactionR\x0btransaction\x12+\n\x11\x63ompletion_offset\x18\x02 \x01(\tR\x10\x63ompletionOffset\"\xa1\x01\n\'SubmitAndWaitForTransactionTreeResponse\x12I\n\x0btransaction\x18\x01 \x01(\x0b\x32\'.com.daml.ledger.api.v1.TransactionTreeR\x0btransaction\x12+\n\x11\x63ompletion_offset\x18\x02 \x01(\tR\x10\x63ompletionOffset2\x94\x04\n\x0e\x43ommandService\x12U\n\rSubmitAndWait\x12,.com.daml.ledger.api.v1.SubmitAndWaitRequest\x1a\x16.google.protobuf.Empty\x12\x8c\x01\n\x1dSubmitAndWaitForTransactionId\x12,.com.daml.ledger.api.v1.SubmitAndWaitRequest\x1a=.com.daml.ledger.api.v1.SubmitAndWaitForTransactionIdResponse\x12\x88\x01\n\x1bSubmitAndWaitForTransaction\x12,.com.daml.ledger.api.v1.SubmitAndWaitRequest\x1a;.com.daml.ledger.api.v1.SubmitAndWaitForTransactionResponse\x12\x90\x01\n\x1fSubmitAndWaitForTransactionTree\x12,.com.daml.ledger.api.v1.SubmitAndWaitRequest\x1a?.com.daml.ledger.api.v1.SubmitAndWaitForTransactionTreeResponseB\x92\x01\n\x16\x63om.daml.ledger.api.v1B\x18\x43ommandServiceOuterClassZEgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1\xaa\x02\x16\x43om.Daml.Ledger.Api.V1b\x06proto3')



_SUBMITANDWAITREQUEST = DESCRIPTOR.message_types_by_name['SubmitAndWaitRequest']
_SUBMITANDWAITFORTRANSACTIONIDRESPONSE = DESCRIPTOR.message_types_by_name['SubmitAndWaitForTransactionIdResponse']
_SUBMITANDWAITFORTRANSACTIONRESPONSE = DESCRIPTOR.message_types_by_name['SubmitAndWaitForTransactionResponse']
_SUBMITANDWAITFORTRANSACTIONTREERESPONSE = DESCRIPTOR.message_types_by_name['SubmitAndWaitForTransactionTreeResponse']
SubmitAndWaitRequest = _reflection.GeneratedProtocolMessageType('SubmitAndWaitRequest', (_message.Message,), {
  'DESCRIPTOR' : _SUBMITANDWAITREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.command_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.SubmitAndWaitRequest)
  })
_sym_db.RegisterMessage(SubmitAndWaitRequest)

SubmitAndWaitForTransactionIdResponse = _reflection.GeneratedProtocolMessageType('SubmitAndWaitForTransactionIdResponse', (_message.Message,), {
  'DESCRIPTOR' : _SUBMITANDWAITFORTRANSACTIONIDRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.command_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.SubmitAndWaitForTransactionIdResponse)
  })
_sym_db.RegisterMessage(SubmitAndWaitForTransactionIdResponse)

SubmitAndWaitForTransactionResponse = _reflection.GeneratedProtocolMessageType('SubmitAndWaitForTransactionResponse', (_message.Message,), {
  'DESCRIPTOR' : _SUBMITANDWAITFORTRANSACTIONRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.command_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.SubmitAndWaitForTransactionResponse)
  })
_sym_db.RegisterMessage(SubmitAndWaitForTransactionResponse)

SubmitAndWaitForTransactionTreeResponse = _reflection.GeneratedProtocolMessageType('SubmitAndWaitForTransactionTreeResponse', (_message.Message,), {
  'DESCRIPTOR' : _SUBMITANDWAITFORTRANSACTIONTREERESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.command_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.SubmitAndWaitForTransactionTreeResponse)
  })
_sym_db.RegisterMessage(SubmitAndWaitForTransactionTreeResponse)

_COMMANDSERVICE = DESCRIPTOR.services_by_name['CommandService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026com.daml.ledger.api.v1B\030CommandServiceOuterClassZEgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1\252\002\026Com.Daml.Ledger.Api.V1'
  _SUBMITANDWAITREQUEST._serialized_start=182
  _SUBMITANDWAITREQUEST._serialized_end=266
  _SUBMITANDWAITFORTRANSACTIONIDRESPONSE._serialized_start=268
  _SUBMITANDWAITFORTRANSACTIONIDRESPONSE._serialized_end=391
  _SUBMITANDWAITFORTRANSACTIONRESPONSE._serialized_start=394
  _SUBMITANDWAITFORTRANSACTIONRESPONSE._serialized_end=547
  _SUBMITANDWAITFORTRANSACTIONTREERESPONSE._serialized_start=550
  _SUBMITANDWAITFORTRANSACTIONTREERESPONSE._serialized_end=711
  _COMMANDSERVICE._serialized_start=714
  _COMMANDSERVICE._serialized_end=1246
# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

from datetime import date, datetime
from decimal import Decimal
import json

from .contracts import ContractId
from .datetime import date_to_str, datetime_to_str

__all__ = ["JSONEncoder"]


class JSONEncoder(json.JSONEncoder):
    """
    Convert some known Ledger API primitive types into their appropriate JSON representations.
    """

    def default(self, o):
        if isinstance(o, datetime):
            return datetime_to_str(o)
        elif isinstance(o, date):
            return date_to_str(o)
        elif isinstance(o, ContractId):
            return o.value
        elif isinstance(o, Decimal):
            return str(o)
        elif isinstance(o, UnquotedDecimal):
            return UnquotedDecimal(o)

        return json.JSONEncoder.default(self, o)


class UnquotedDecimal(float):
    """
    A specialized number that is used just for JSON serialization; it suppresses quotes that would
    otherwise appear if ``JSONEncoder.default`` returned a string where it intended to use a number.
    """

    def __new__(cls, decimal_value):
        # noinspection PyArgumentList
        return float.__new__(cls, float(decimal_value))

    def __init__(self, decimal_value):
        float.__init__(float(decimal_value))
        self.decimal_value = decimal_value

    def __repr__(self):
        return str(self.decimal_value)



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file

import builtins as _builtins, sys, typing as _typing

from google.protobuf.message import Message as _Message

from .experimental_features_pb2 import ExperimentalFeatures

if sys.version_info >= (3, 8):
    from typing import Literal as _L
else:
    from typing_extensions import Literal as _L

__all__ = [
    "GetLedgerApiVersionRequest",
    "GetLedgerApiVersionResponse",
    "FeaturesDescriptor",
    "UserManagementFeature",
]


class GetLedgerApiVersionRequest(_Message):
    ledger_id: _builtins.str
    def __init__(self, *, ledger_id: _typing.Optional[_builtins.str] = ...): ...
    def HasField(self, field_name: _L["ledger_id"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["ledger_id"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class GetLedgerApiVersionResponse(_Message):
    version: _builtins.str
    @property
    def features(self) -> FeaturesDescriptor: ...
    def __init__(self, *, version: _typing.Optional[_builtins.str] = ..., features: _typing.Optional[FeaturesDescriptor] = ...): ...
    def HasField(self, field_name: _L["version", "features"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["version", "features"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class FeaturesDescriptor(_Message):
    @property
    def user_management(self) -> UserManagementFeature: ...
    @property
    def experimental(self) -> ExperimentalFeatures: ...
    def __init__(self, *, user_management: _typing.Optional[UserManagementFeature] = ..., experimental: _typing.Optional[ExperimentalFeatures] = ...): ...
    def HasField(self, field_name: _L["user_management", "experimental"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["user_management", "experimental"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class UserManagementFeature(_Message):
    supported: _builtins.bool
    max_rights_per_user: _builtins.int
    max_users_page_size: _builtins.int
    def __init__(self, *, supported: _typing.Optional[_builtins.bool] = ..., max_rights_per_user: _typing.Optional[_builtins.int] = ..., max_users_page_size: _typing.Optional[_builtins.int] = ...): ...
    def HasField(self, field_name: _L["supported", "max_rights_per_user", "max_users_page_size"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["supported", "max_rights_per_user", "max_users_page_size"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file

import builtins as _builtins, sys, typing as _typing

from google.protobuf.duration_pb2 import Duration
from google.protobuf.message import Message as _Message
from google.protobuf.timestamp_pb2 import Timestamp

if sys.version_info >= (3, 8):
    from typing import Literal as _L
else:
    from typing_extensions import Literal as _L

__all__ = [
    "GetTimeModelRequest",
    "GetTimeModelResponse",
    "SetTimeModelRequest",
    "SetTimeModelResponse",
    "TimeModel",
]


class GetTimeModelRequest(_Message):
    def __init__(self): ...
    def HasField(self, field_name: _typing.NoReturn) -> _typing.NoReturn: ...
    def ClearField(self, field_name: _typing.NoReturn) -> _typing.NoReturn: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class GetTimeModelResponse(_Message):
    configuration_generation: _builtins.int
    @property
    def time_model(self) -> TimeModel: ...
    def __init__(self, *, configuration_generation: _typing.Optional[_builtins.int] = ..., time_model: _typing.Optional[TimeModel] = ...): ...
    def HasField(self, field_name: _L["configuration_generation", "time_model"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["configuration_generation", "time_model"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class SetTimeModelRequest(_Message):
    submission_id: _builtins.str
    @property
    def maximum_record_time(self) -> Timestamp: ...
    configuration_generation: _builtins.int
    @property
    def new_time_model(self) -> TimeModel: ...
    def __init__(self, *, submission_id: _typing.Optional[_builtins.str] = ..., maximum_record_time: _typing.Optional[Timestamp] = ..., configuration_generation: _typing.Optional[_builtins.int] = ..., new_time_model: _typing.Optional[TimeModel] = ...): ...
    def HasField(self, field_name: _L["submission_id", "maximum_record_time", "configuration_generation", "new_time_model"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["submission_id", "maximum_record_time", "configuration_generation", "new_time_model"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class SetTimeModelResponse(_Message):
    configuration_generation: _builtins.int
    def __init__(self, *, configuration_generation: _typing.Optional[_builtins.int] = ...): ...
    def HasField(self, field_name: _L["configuration_generation"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["configuration_generation"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class TimeModel(_Message):
    @property
    def avg_transaction_latency(self) -> Duration: ...
    @property
    def min_skew(self) -> Duration: ...
    @property
    def max_skew(self) -> Duration: ...
    def __init__(self, *, avg_transaction_latency: _typing.Optional[Duration] = ..., min_skew: _typing.Optional[Duration] = ..., max_skew: _typing.Optional[Duration] = ...): ...
    def HasField(self, field_name: _L["avg_transaction_latency", "min_skew", "max_skew"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["avg_transaction_latency", "min_skew", "max_skew"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file

import builtins as _builtins, sys, typing as _typing

from google.protobuf.internal.containers import MessageMap, RepeatedCompositeFieldContainer, RepeatedScalarFieldContainer
from google.protobuf.message import Message as _Message
from google.protobuf.timestamp_pb2 import Timestamp

from .event_pb2 import CreatedEvent, Event, ExercisedEvent

if sys.version_info >= (3, 8):
    from typing import Literal as _L
else:
    from typing_extensions import Literal as _L

__all__ = [
    "TransactionTree",
    "TreeEvent",
    "Transaction",
]


class TransactionTree(_Message):
    transaction_id: _builtins.str
    command_id: _builtins.str
    workflow_id: _builtins.str
    @property
    def effective_at(self) -> Timestamp: ...
    offset: _builtins.str
    @property
    def events_by_id(self) -> MessageMap[_builtins.str, TreeEvent]: ...
    @property
    def root_event_ids(self) -> RepeatedScalarFieldContainer[_builtins.str]: ...
    def __init__(self, *, transaction_id: _typing.Optional[_builtins.str] = ..., command_id: _typing.Optional[_builtins.str] = ..., workflow_id: _typing.Optional[_builtins.str] = ..., effective_at: _typing.Optional[Timestamp] = ..., offset: _typing.Optional[_builtins.str] = ..., events_by_id: _typing.Optional[_typing.Mapping[_builtins.str, TreeEvent]] = ..., root_event_ids: _typing.Optional[_typing.Iterable[_builtins.str]] = ...): ...
    def HasField(self, field_name: _L["transaction_id", "command_id", "workflow_id", "effective_at", "offset", "events_by_id", "root_event_ids"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["transaction_id", "command_id", "workflow_id", "effective_at", "offset", "events_by_id", "root_event_ids"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class TreeEvent(_Message):
    @property
    def created(self) -> CreatedEvent: ...
    @property
    def exercised(self) -> ExercisedEvent: ...
    @_typing.overload
    def __init__(self): ...
    @_typing.overload
    def __init__(self, *, created: CreatedEvent = ...): ...
    @_typing.overload
    def __init__(self, *, exercised: ExercisedEvent = ...): ...
    def HasField(self, field_name: _L["kind", "created", "exercised"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["kind", "created", "exercised"]) -> None: ...
    def WhichOneof(self, oneof_group: _L["kind"]) -> _L[None, "created", "exercised"]: ...

class Transaction(_Message):
    transaction_id: _builtins.str
    command_id: _builtins.str
    workflow_id: _builtins.str
    @property
    def effective_at(self) -> Timestamp: ...
    @property
    def events(self) -> RepeatedCompositeFieldContainer[Event]: ...
    offset: _builtins.str
    def __init__(self, *, transaction_id: _typing.Optional[_builtins.str] = ..., command_id: _typing.Optional[_builtins.str] = ..., workflow_id: _typing.Optional[_builtins.str] = ..., effective_at: _typing.Optional[Timestamp] = ..., events: _typing.Optional[_typing.Iterable[Event]] = ..., offset: _typing.Optional[_builtins.str] = ...): ...
    def HasField(self, field_name: _L["transaction_id", "command_id", "workflow_id", "effective_at", "events", "offset"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["transaction_id", "command_id", "workflow_id", "effective_at", "events", "offset"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

from typing import Optional, Union

from ..damlast import TypeConName
from ..ledger import CreateEvent
from ..ledger.aio import Connection
from ..query import Query


async def fetch_first(
    __conn: Connection, __template_id: Union[str, TypeConName], __query: Query = None
) -> Optional[CreateEvent]:
    """
    Return the first (earliest) :class:`CreateEvent` from the Active Contract Set (ACS), or ``None``
    if no contract is currently a part of the ACS.

    :param __conn: The connection to use to fetch results.
    :param __template_id: The ID of the template to fetch.
    :param __query: An optional query to further filter results.
    :return:
        The first :class:`CreateEvent`, or ``None`` if no matching :class:`CreateEvent` currently
        exists.
    """
    async with __conn.query(__template_id, __query) as stream:
        async for event in stream.creates():
            return event

    return None


async def fetch_last(
    __conn: Connection, __template_id: Union[str, TypeConName], __query: Query = None
) -> Optional[CreateEvent]:
    """
    Return the last (most recent) :class:`CreateEvent` from the Active Contract Set (ACS), or
    ``None`` if no contract is currently a part of the ACS.

    :param __conn: The connection to use to fetch results.
    :param __template_id: The ID of the template to fetch.
    :param __query: An optional query to further filter results.
    :return:
        The last :class:`CreateEvent`, or ``None`` if no matching :class:`CreateEvent` currently
        exists.
    """
    last_event = None
    async with __conn.query(__template_id, __query) as stream:
        async for event in stream.creates():
            last_event = event

    return last_event



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file

import builtins as _builtins, sys, typing as _typing

from google.protobuf.empty_pb2 import Empty
from google.protobuf.internal.containers import RepeatedCompositeFieldContainer
from google.protobuf.message import Message as _Message

if sys.version_info >= (3, 8):
    from typing import Literal as _L
else:
    from typing_extensions import Literal as _L

__all__ = [
    "Value",
    "Record",
    "RecordField",
    "Identifier",
    "Variant",
    "Enum",
    "List",
    "Optional",
    "Map",
    "GenMap",
]


class Value(_Message):
    @property
    def record(self) -> Record: ...
    @property
    def variant(self) -> Variant: ...
    contract_id: _builtins.str
    @property
    def list(self) -> List: ...
    int64: _builtins.int
    numeric: _builtins.str
    text: _builtins.str
    timestamp: _builtins.float
    party: _builtins.str
    bool: _builtins.bool
    @property
    def unit(self) -> Empty: ...
    date: _builtins.int
    @property
    def optional(self) -> Optional: ...
    @property
    def map(self) -> Map: ...
    @property
    def enum(self) -> Enum: ...
    @property
    def gen_map(self) -> GenMap: ...
    @_typing.overload
    def __init__(self): ...
    @_typing.overload
    def __init__(self, *, record: Record = ...): ...
    @_typing.overload
    def __init__(self, *, variant: Variant = ...): ...
    @_typing.overload
    def __init__(self, *, contract_id: _builtins.str = ...): ...
    @_typing.overload
    def __init__(self, *, list: List = ...): ...
    @_typing.overload
    def __init__(self, *, int64: _builtins.int = ...): ...
    @_typing.overload
    def __init__(self, *, numeric: _builtins.str = ...): ...
    @_typing.overload
    def __init__(self, *, text: _builtins.str = ...): ...
    @_typing.overload
    def __init__(self, *, timestamp: _builtins.float = ...): ...
    @_typing.overload
    def __init__(self, *, party: _builtins.str = ...): ...
    @_typing.overload
    def __init__(self, *, bool: _builtins.bool = ...): ...
    @_typing.overload
    def __init__(self, *, unit: Empty = ...): ...
    @_typing.overload
    def __init__(self, *, date: _builtins.int = ...): ...
    @_typing.overload
    def __init__(self, *, optional: Optional = ...): ...
    @_typing.overload
    def __init__(self, *, map: Map = ...): ...
    @_typing.overload
    def __init__(self, *, enum: Enum = ...): ...
    @_typing.overload
    def __init__(self, *, gen_map: GenMap = ...): ...
    def HasField(self, field_name: _L["Sum", "record", "variant", "contract_id", "list", "int64", "numeric", "text", "timestamp", "party", "bool", "unit", "date", "optional", "map", "enum", "gen_map"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["Sum", "record", "variant", "contract_id", "list", "int64", "numeric", "text", "timestamp", "party", "bool", "unit", "date", "optional", "map", "enum", "gen_map"]) -> None: ...
    def WhichOneof(self, oneof_group: _L["Sum"]) -> _L[None, "record", "variant", "contract_id", "list", "int64", "numeric", "text", "timestamp", "party", "bool", "unit", "date", "optional", "map", "enum", "gen_map"]: ...

class Record(_Message):
    @property
    def record_id(self) -> Identifier: ...
    @property
    def fields(self) -> RepeatedCompositeFieldContainer[RecordField]: ...
    def __init__(self, *, record_id: _typing.Optional[Identifier] = ..., fields: _typing.Optional[_typing.Iterable[RecordField]] = ...): ...
    def HasField(self, field_name: _L["record_id", "fields"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["record_id", "fields"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class RecordField(_Message):
    label: _builtins.str
    @property
    def value(self) -> Value: ...
    def __init__(self, *, label: _typing.Optional[_builtins.str] = ..., value: _typing.Optional[Value] = ...): ...
    def HasField(self, field_name: _L["label", "value"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["label", "value"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class Identifier(_Message):
    package_id: _builtins.str
    module_name: _builtins.str
    entity_name: _builtins.str
    def __init__(self, *, package_id: _typing.Optional[_builtins.str] = ..., module_name: _typing.Optional[_builtins.str] = ..., entity_name: _typing.Optional[_builtins.str] = ...): ...
    def HasField(self, field_name: _L["package_id", "module_name", "entity_name"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["package_id", "module_name", "entity_name"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class Variant(_Message):
    @property
    def variant_id(self) -> Identifier: ...
    constructor: _builtins.str
    @property
    def value(self) -> Value: ...
    def __init__(self, *, variant_id: _typing.Optional[Identifier] = ..., constructor: _typing.Optional[_builtins.str] = ..., value: _typing.Optional[Value] = ...): ...
    def HasField(self, field_name: _L["variant_id", "constructor", "value"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["variant_id", "constructor", "value"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class Enum(_Message):
    @property
    def enum_id(self) -> Identifier: ...
    constructor: _builtins.str
    def __init__(self, *, enum_id: _typing.Optional[Identifier] = ..., constructor: _typing.Optional[_builtins.str] = ...): ...
    def HasField(self, field_name: _L["enum_id", "constructor"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["enum_id", "constructor"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class List(_Message):
    @property
    def elements(self) -> RepeatedCompositeFieldContainer[Value]: ...
    def __init__(self, *, elements: _typing.Optional[_typing.Iterable[Value]] = ...): ...
    def HasField(self, field_name: _L["elements"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["elements"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class Optional(_Message):
    @property
    def value(self) -> Value: ...
    def __init__(self, *, value: _typing.Optional[Value] = ...): ...
    def HasField(self, field_name: _L["value"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["value"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class Map(_Message):
    
    class Entry(_Message):
        key: _builtins.str
        @property
        def value(self) -> Value: ...
        def __init__(self, *, key: _typing.Optional[_builtins.str] = ..., value: _typing.Optional[Value] = ...): ...
        def HasField(self, field_name: _L["key", "value"]) -> _builtins.bool: ...
        def ClearField(self, field_name: _L["key", "value"]) -> None: ...
        def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...
    @property
    def entries(self) -> RepeatedCompositeFieldContainer[Map.Entry]: ...
    def __init__(self, *, entries: _typing.Optional[_typing.Iterable[Map.Entry]] = ...): ...
    def HasField(self, field_name: _L["entries"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["entries"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class GenMap(_Message):
    
    class Entry(_Message):
        @property
        def key(self) -> Value: ...
        @property
        def value(self) -> Value: ...
        def __init__(self, *, key: _typing.Optional[Value] = ..., value: _typing.Optional[Value] = ...): ...
        def HasField(self, field_name: _L["key", "value"]) -> _builtins.bool: ...
        def ClearField(self, field_name: _L["key", "value"]) -> None: ...
        def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...
    @property
    def entries(self) -> RepeatedCompositeFieldContainer[GenMap.Entry]: ...
    def __init__(self, *, entries: _typing.Optional[_typing.Iterable[GenMap.Entry]] = ...): ...
    def HasField(self, field_name: _L["entries"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["entries"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/ledger_identity_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n4com/daml/ledger/api/v1/ledger_identity_service.proto\x12\x16\x63om.daml.ledger.api.v1\"\x1e\n\x18GetLedgerIdentityRequest:\x02\x18\x01\"@\n\x19GetLedgerIdentityResponse\x12\x1f\n\tledger_id\x18\x01 \x01(\tB\x02\x18\x01R\x08ledgerId:\x02\x18\x01\x32\x9b\x01\n\x15LedgerIdentityService\x12}\n\x11GetLedgerIdentity\x12\x30.com.daml.ledger.api.v1.GetLedgerIdentityRequest\x1a\x31.com.daml.ledger.api.v1.GetLedgerIdentityResponse\"\x03\x88\x02\x01\x1a\x03\x88\x02\x01\x42\x99\x01\n\x16\x63om.daml.ledger.api.v1B\x1fLedgerIdentityServiceOuterClassZEgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1\xaa\x02\x16\x43om.Daml.Ledger.Api.V1b\x06proto3')



_GETLEDGERIDENTITYREQUEST = DESCRIPTOR.message_types_by_name['GetLedgerIdentityRequest']
_GETLEDGERIDENTITYRESPONSE = DESCRIPTOR.message_types_by_name['GetLedgerIdentityResponse']
GetLedgerIdentityRequest = _reflection.GeneratedProtocolMessageType('GetLedgerIdentityRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETLEDGERIDENTITYREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.ledger_identity_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetLedgerIdentityRequest)
  })
_sym_db.RegisterMessage(GetLedgerIdentityRequest)

GetLedgerIdentityResponse = _reflection.GeneratedProtocolMessageType('GetLedgerIdentityResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETLEDGERIDENTITYRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.ledger_identity_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetLedgerIdentityResponse)
  })
_sym_db.RegisterMessage(GetLedgerIdentityResponse)

_LEDGERIDENTITYSERVICE = DESCRIPTOR.services_by_name['LedgerIdentityService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026com.daml.ledger.api.v1B\037LedgerIdentityServiceOuterClassZEgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1\252\002\026Com.Daml.Ledger.Api.V1'
  _GETLEDGERIDENTITYREQUEST._options = None
  _GETLEDGERIDENTITYREQUEST._serialized_options = b'\030\001'
  _GETLEDGERIDENTITYRESPONSE.fields_by_name['ledger_id']._options = None
  _GETLEDGERIDENTITYRESPONSE.fields_by_name['ledger_id']._serialized_options = b'\030\001'
  _GETLEDGERIDENTITYRESPONSE._options = None
  _GETLEDGERIDENTITYRESPONSE._serialized_options = b'\030\001'
  _LEDGERIDENTITYSERVICE._options = None
  _LEDGERIDENTITYSERVICE._serialized_options = b'\210\002\001'
  _LEDGERIDENTITYSERVICE.methods_by_name['GetLedgerIdentity']._options = None
  _LEDGERIDENTITYSERVICE.methods_by_name['GetLedgerIdentity']._serialized_options = b'\210\002\001'
  _GETLEDGERIDENTITYREQUEST._serialized_start=80
  _GETLEDGERIDENTITYREQUEST._serialized_end=110
  _GETLEDGERIDENTITYRESPONSE._serialized_start=112
  _GETLEDGERIDENTITYRESPONSE._serialized_end=176
  _LEDGERIDENTITYSERVICE._serialized_start=179
  _LEDGERIDENTITYSERVICE._serialized_end=334
# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
This module contains implementations for the different protocols and serialization formats
supported by this client library.
"""

from ._base import LedgerClient, LedgerConnectionOptions, LedgerNetwork
from .v1 import grpc



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
Utilities for running protoc, and easily composing Protobuf calls in python.

To create a Protobuf plugin in Python, define an entrypoint as so:

.. code-block:: python

   @protoc_plugin
   def main(request: CodeGeneratorRequest) -> CodeGeneratorResponse:
       # This does what `python3 -m grpc_tools.protoc ...` does,
       # but as a function call in Python instead of an awkward exec
       python_response = run_plugin('python', request)

       # It is typical for a plugin to need to ultimately call one of the
       # built-in plugins (for example, to post-process the data from a built-in plugin).
       for f in python_response.file:
           f.proto_file = COPYRIGHT_NOTICE + f.proto_file

       response = CodeGeneratorResponse()
       response.MergeFrom(python_response)
       return response
"""

from functools import wraps
import os
from pathlib import Path
import site
import subprocess
import sys
from tempfile import TemporaryDirectory
from traceback import print_exc
from typing import Callable, List, NoReturn

from google.protobuf.compiler.plugin_pb2 import CodeGeneratorRequest, CodeGeneratorResponse
from google.protobuf.descriptor_pb2 import FileDescriptorSet
from grpc_tools import protoc as _protoc
import pkg_resources

__all__ = ["main", "protoc_plugin", "run_plugin"]


def main() -> "NoReturn":
    """
    Entrypoint that runs ``protoc``, but with locally exported binary plugins, and automatically
    including Protobuf files that are built into ``grpc_tools`` and that have been pulled through
    as Python dependencies.
    """
    sys.exit(_main(sys.argv))


def _main(argv):
    os.environ["PATH"] = f'{os.getenv("PATH")}:{os.path.dirname(sys.executable)}'

    includes = [
        pkg_resources.resource_filename("grpc_tools", "_proto"),
        *site.getsitepackages(),
    ]
    return _protoc.main(argv + ["-I" + inc for inc in includes])


def protoc_plugin(
    fn: "Callable[[CodeGeneratorRequest], CodeGeneratorResponse]",
) -> "Callable[[], NoReturn]":
    """
    Decorator that converts a callable function to a ``protoc`` plugin.

    This function does NOT return, and is intended to be used as the main function for a Python
    program/script.
    """

    @wraps(fn)
    def _body() -> "NoReturn":
        try:
            data = sys.stdin.buffer.read()
            request = CodeGeneratorRequest.FromString(data)
            response = fn(request)
            code = 0
        except Exception as ex:
            print_exc(file=sys.stderr)
            response = CodeGeneratorResponse(error=str(ex))
            code = 1

        sys.stdout.buffer.write(response.SerializeToString())
        sys.exit(code)

    return _body


def run_plugin(plugin_name: "str", request: "CodeGeneratorRequest") -> "CodeGeneratorResponse":
    if plugin_name in ("python", "grpc_python"):
        return run_plugin_built_in(plugin_name, request)
    else:
        return run_plugin_external(plugin_name, request)


def run_plugin_built_in(
    plugin_name: "str", request: "CodeGeneratorRequest"
) -> "CodeGeneratorResponse":
    with TemporaryDirectory() as tmpdir:
        input_file = Path(tmpdir) / "in.bin"
        output_dir = Path(tmpdir) / "out"

        output_dir.mkdir()

        input_file.write_bytes(FileDescriptorSet(file=request.proto_file).SerializeToString())

        # re-run the built-in code generator, but with the arguments that we were given
        invocation = [
            "_dazl_pb.protoc",
            f"--descriptor_set_in={input_file}",
            f"--{plugin_name}_out={output_dir}",
            *request.file_to_generate,
        ]

        exit_code = _main(invocation)
        if exit_code:
            raise Exception(f"failed to run {invocation} (exit code {exit_code})")

        files = [
            CodeGeneratorResponse.File(name=str(p.relative_to(output_dir)), content=p.read_text())
            for p in output_dir.rglob("*")
            if p.is_file()
        ]
        return CodeGeneratorResponse(file=files)


def run_plugin_external(
    plugin_name: "str", request: "CodeGeneratorRequest"
) -> "CodeGeneratorResponse":
    args = ["protoc-gen-" + plugin_name]  # type: List[str]
    proc = subprocess.run(args, input=request.SerializeToString(), capture_output=True)  # type: ignore

    response = CodeGeneratorResponse()
    response.ParseFromString(proc.stdout)
    return response



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
The :mod:`dazl.util.prim_types` module contains functions for converting arbitrary objects and
on-wire formats to/from canonical Python types.
"""
from datetime import date, datetime, timedelta
from decimal import Decimal
from typing import Any, Mapping, Optional, Tuple, Union, overload
import warnings

from ..prim.datetime import DATETIME_ISO8601_Z_FORMAT, TimeDeltaLike as TimeDeltaConvertible

warnings.warn(
    "dazl.util.prim_types is deprecated; use dazl.prim instead.", DeprecationWarning, stacklevel=2
)

__all__ = [
    "to_boolean",
    "to_str",
    "to_int",
    "to_date",
    "to_datetime",
    "to_decimal",
    "to_ledger_api_decimal",
    "to_timedelta",
    "PrimitiveTypeConverter",
    "decode_variant_dict",
    "standard_format",
    "nano_format",
    "unflatten_dotted_keys",
    "DEFAULT_TYPE_CONVERTER",
    "DATETIME_ISO8601_Z_FORMAT",
    "frozendict",
    "to_hashable",
]


@overload
def to_boolean(obj: "Union[bool, str]") -> bool:
    ...


@overload
def to_boolean(obj: None) -> None:
    ...


def to_boolean(obj):
    from ..prim import to_bool

    warnings.warn(
        "dazl.util.prim_types.to_boolean is deprecated; use dazl.prim.to_bool instead.",
        DeprecationWarning,
        stacklevel=2,
    )
    return to_bool(obj) if obj is not None else None


def to_str(obj: "Optional[Any]") -> "Optional[str]":
    from ..prim import to_str

    warnings.warn(
        "dazl.util.prim_types.to_str is deprecated; use dazl.prim.to_str instead.",
        DeprecationWarning,
        stacklevel=2,
    )
    return to_str(obj) if obj is not None else None


def to_int(obj: Any) -> "Optional[int]":
    from ..prim import to_int

    warnings.warn(
        "dazl.util.prim_types.to_int is deprecated; use dazl.prim.to_int instead.",
        DeprecationWarning,
        stacklevel=2,
    )
    return to_int(obj) if obj is not None else None


def to_date(obj: "Optional[Any]") -> "Optional[date]":
    from ..prim import to_date

    warnings.warn(
        "dazl.util.prim_types.to_date is deprecated; use dazl.prim.to_date instead.",
        DeprecationWarning,
        stacklevel=2,
    )
    return to_date(obj) if obj is not None else None


def to_datetime(obj: "Optional[Any]") -> "Optional[datetime]":
    from ..prim import to_datetime

    warnings.warn(
        "dazl.util.prim_types.to_datetime is deprecated; use dazl.prim.to_datetime instead.",
        DeprecationWarning,
        stacklevel=2,
    )
    return to_datetime(obj) if obj is not None else None


def to_decimal(obj: "Optional[Any]") -> "Optional[Decimal]":
    from ..prim import to_decimal

    warnings.warn(
        "dazl.util.prim_types.to_decimal is deprecated; use dazl.prim.to_decimal instead.",
        DeprecationWarning,
        stacklevel=2,
    )
    return to_decimal(obj) if obj is not None else None


def to_ledger_api_decimal(obj: "Decimal") -> str:
    from ..prim import decimal_to_str

    warnings.warn(
        "dazl.util.prim_types.to_ledger_api_decimal is deprecated; use dazl.prim.decimal_to_str instead.",
        DeprecationWarning,
        stacklevel=2,
    )
    return decimal_to_str(obj)


def to_timedelta(obj: TimeDeltaConvertible) -> "timedelta":
    from ..prim import to_timedelta

    warnings.warn(
        "dazl.util.prim_types.to_timedelta is deprecated; use dazl.prim.to_timedelta instead.",
        DeprecationWarning,
        stacklevel=2,
    )
    return to_timedelta(obj)


class PrimitiveTypeConverter:
    to_boolean = staticmethod(to_boolean)
    to_str = staticmethod(to_str)
    to_int = staticmethod(to_int)
    to_decimal = staticmethod(to_decimal)
    to_date = staticmethod(to_date)
    to_datetime = staticmethod(to_datetime)
    to_timedelta = staticmethod(to_timedelta)


def decode_variant_dict(obj: "Any") -> "Tuple[str, Any]":
    from ..prim import to_variant

    warnings.warn(
        "dazl.util.prim_types.decode_variant_dict is deprecated; use dazl.prim.to_variant instead.",
        DeprecationWarning,
        stacklevel=2,
    )
    return to_variant(obj)


def standard_format(fmt, value):
    from ..prim.datetime import _parse

    warnings.warn(
        "dazl.util.prim_types.nano_format is deprecated; there is no replacement.",
        DeprecationWarning,
        stacklevel=2,
    )
    return _parse(fmt, value)


def nano_format(value):
    from ..prim.datetime import _parse_nano_format

    warnings.warn(
        "dazl.util.prim_types.nano_format is deprecated; there is no replacement.",
        DeprecationWarning,
        stacklevel=2,
    )
    return _parse_nano_format(value)


def unflatten_dotted_keys(m: "Mapping[str, Any]"):
    from ..prim import to_record

    warnings.warn(
        "dazl.util.prim_types.unflatten_dotted_keys is deprecated; use dazl.prim.to_record instead.",
        DeprecationWarning,
        stacklevel=2,
    )
    return to_record(m) if m is not None else None


DEFAULT_TYPE_CONVERTER = PrimitiveTypeConverter()


def frozendict(*args, **kwargs):
    from ..prim.map import FrozenDict

    warnings.warn(
        "dazl.util.prim_types.frozendict is deprecated; use dazl.prim.map.FrozenDict instead.",
        DeprecationWarning,
        stacklevel=2,
    )
    return FrozenDict(*args, **kwargs)


def to_hashable(*args, **kwargs):
    from ..prim.map import to_hashable

    warnings.warn(
        "dazl.util.prim_types.to_hashable is deprecated; use dazl.prim.map.to_hashable instead.",
        DeprecationWarning,
        stacklevel=2,
    )
    return to_hashable(*args, **kwargs)



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
This model contains definitions for the data classes used at the client layer of this library.
"""

from enum import IntEnum
import sys
from typing import NamedTuple, Optional

if sys.version_info >= (3, 8):
    from typing import Literal
else:
    from typing_extensions import Literal

__all__ = [
    "ExitCode",
    "LedgerRun",
    "CREATE_IF_MISSING",
    "NONE_IF_MISSING",
    "EXCEPTION_IF_MISSING",
    "IfMissingPartyBehavior",
]


class ExitCode(IntEnum):
    """
    The different ways that a run of a dazl script can terminate.
    """

    SUCCESS = 0
    ABORT = 1
    ERROR = 2
    ERROR_INIT = 3


class LedgerRun(NamedTuple):
    """
    The results of a run of the ledger.

    Instance attributes:

    .. attribute:: LedgerRun.exit_code

        The exit code.

    .. attribute:: LedgerRun.block_start_height

        The block height of the first block consumed by the run.

    .. attribute:: LedgerRun.block_end_height

        The block height of the first block NOT consumed by the run.
    """

    exit_code: ExitCode
    block_start_height: Optional[int]
    block_end_height: Optional[int]


CREATE_IF_MISSING: Literal[1] = 1
NONE_IF_MISSING: Literal[2] = 2
EXCEPTION_IF_MISSING: Literal[3] = 3

IfMissingPartyBehavior = Literal[1, 2, 3]



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

from os import PathLike
from typing import Collection, Optional, Union

from ...damlast.lookup import MultiPackageLookup
from ...prim import Party, TimeDeltaLike
from .conn_aio import Connection

__all__ = ["connect", "Connection"]

# TODO: Figure out clever ways to make this function's type signature easier to maintain while
#  preserving its ease of use to callers.
def connect(
    url: str,
    *,
    read_as: "Union[None, Party, Collection[Party]]" = None,
    act_as: "Union[None, Party, Collection[Party]]" = None,
    admin: "Optional[bool]" = False,
    ledger_id: "Optional[str]" = None,
    application_name: "Optional[str]" = None,
    oauth_token: "Optional[str]" = None,
    ca: "Optional[bytes]" = None,
    ca_file: "Optional[PathLike]" = None,
    cert: "Optional[bytes]" = None,
    cert_file: "Optional[PathLike]" = None,
    cert_key: "Optional[bytes]" = None,
    cert_key_file: "Optional[PathLike]" = None,
    connect_timeout: "Optional[TimeDeltaLike]" = None,
    enable_http_proxy: "bool" = True,
    lookup: "Optional[MultiPackageLookup]" = None,
) -> Connection: ...



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file

import builtins as _builtins, sys, typing as _typing

from google.protobuf.internal.containers import RepeatedCompositeFieldContainer, RepeatedScalarFieldContainer
from google.protobuf.message import Message as _Message

from .ledger_offset_pb2 import LedgerOffset
from .transaction_filter_pb2 import TransactionFilter
from .transaction_pb2 import Transaction, TransactionTree

if sys.version_info >= (3, 8):
    from typing import Literal as _L
else:
    from typing_extensions import Literal as _L

__all__ = [
    "GetTransactionsRequest",
    "GetTransactionsResponse",
    "GetTransactionTreesResponse",
    "GetTransactionByEventIdRequest",
    "GetTransactionByIdRequest",
    "GetTransactionResponse",
    "GetFlatTransactionResponse",
    "GetLedgerEndRequest",
    "GetLedgerEndResponse",
]


class GetTransactionsRequest(_Message):
    ledger_id: _builtins.str
    @property
    def begin(self) -> LedgerOffset: ...
    @property
    def end(self) -> LedgerOffset: ...
    @property
    def filter(self) -> TransactionFilter: ...
    verbose: _builtins.bool
    def __init__(self, *, ledger_id: _typing.Optional[_builtins.str] = ..., begin: _typing.Optional[LedgerOffset] = ..., end: _typing.Optional[LedgerOffset] = ..., filter: _typing.Optional[TransactionFilter] = ..., verbose: _typing.Optional[_builtins.bool] = ...): ...
    def HasField(self, field_name: _L["ledger_id", "begin", "end", "filter", "verbose"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["ledger_id", "begin", "end", "filter", "verbose"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class GetTransactionsResponse(_Message):
    @property
    def transactions(self) -> RepeatedCompositeFieldContainer[Transaction]: ...
    def __init__(self, *, transactions: _typing.Optional[_typing.Iterable[Transaction]] = ...): ...
    def HasField(self, field_name: _L["transactions"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["transactions"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class GetTransactionTreesResponse(_Message):
    @property
    def transactions(self) -> RepeatedCompositeFieldContainer[TransactionTree]: ...
    def __init__(self, *, transactions: _typing.Optional[_typing.Iterable[TransactionTree]] = ...): ...
    def HasField(self, field_name: _L["transactions"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["transactions"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class GetTransactionByEventIdRequest(_Message):
    ledger_id: _builtins.str
    event_id: _builtins.str
    @property
    def requesting_parties(self) -> RepeatedScalarFieldContainer[_builtins.str]: ...
    def __init__(self, *, ledger_id: _typing.Optional[_builtins.str] = ..., event_id: _typing.Optional[_builtins.str] = ..., requesting_parties: _typing.Optional[_typing.Iterable[_builtins.str]] = ...): ...
    def HasField(self, field_name: _L["ledger_id", "event_id", "requesting_parties"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["ledger_id", "event_id", "requesting_parties"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class GetTransactionByIdRequest(_Message):
    ledger_id: _builtins.str
    transaction_id: _builtins.str
    @property
    def requesting_parties(self) -> RepeatedScalarFieldContainer[_builtins.str]: ...
    def __init__(self, *, ledger_id: _typing.Optional[_builtins.str] = ..., transaction_id: _typing.Optional[_builtins.str] = ..., requesting_parties: _typing.Optional[_typing.Iterable[_builtins.str]] = ...): ...
    def HasField(self, field_name: _L["ledger_id", "transaction_id", "requesting_parties"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["ledger_id", "transaction_id", "requesting_parties"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class GetTransactionResponse(_Message):
    @property
    def transaction(self) -> TransactionTree: ...
    def __init__(self, *, transaction: _typing.Optional[TransactionTree] = ...): ...
    def HasField(self, field_name: _L["transaction"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["transaction"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class GetFlatTransactionResponse(_Message):
    @property
    def transaction(self) -> Transaction: ...
    def __init__(self, *, transaction: _typing.Optional[Transaction] = ...): ...
    def HasField(self, field_name: _L["transaction"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["transaction"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class GetLedgerEndRequest(_Message):
    ledger_id: _builtins.str
    def __init__(self, *, ledger_id: _typing.Optional[_builtins.str] = ...): ...
    def HasField(self, field_name: _L["ledger_id"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["ledger_id"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class GetLedgerEndResponse(_Message):
    @property
    def offset(self) -> LedgerOffset: ...
    def __init__(self, *, offset: _typing.Optional[LedgerOffset] = ...): ...
    def HasField(self, field_name: _L["offset"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["offset"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/admin/party_management_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n;com/daml/ledger/api/v1/admin/party_management_service.proto\x12\x1c\x63om.daml.ledger.api.v1.admin\"\x19\n\x17GetParticipantIdRequest\"A\n\x18GetParticipantIdResponse\x12%\n\x0eparticipant_id\x18\x01 \x01(\tR\rparticipantId\"-\n\x11GetPartiesRequest\x12\x18\n\x07parties\x18\x01 \x03(\tR\x07parties\"e\n\x12GetPartiesResponse\x12O\n\rparty_details\x18\x01 \x03(\x0b\x32*.com.daml.ledger.api.v1.admin.PartyDetailsR\x0cpartyDetails\"\x19\n\x17ListKnownPartiesRequest\"k\n\x18ListKnownPartiesResponse\x12O\n\rparty_details\x18\x01 \x03(\x0b\x32*.com.daml.ledger.api.v1.admin.PartyDetailsR\x0cpartyDetails\"]\n\x14\x41llocatePartyRequest\x12\"\n\rparty_id_hint\x18\x01 \x01(\tR\x0bpartyIdHint\x12!\n\x0c\x64isplay_name\x18\x02 \x01(\tR\x0b\x64isplayName\"h\n\x15\x41llocatePartyResponse\x12O\n\rparty_details\x18\x01 \x01(\x0b\x32*.com.daml.ledger.api.v1.admin.PartyDetailsR\x0cpartyDetails\"b\n\x0cPartyDetails\x12\x14\n\x05party\x18\x01 \x01(\tR\x05party\x12!\n\x0c\x64isplay_name\x18\x02 \x01(\tR\x0b\x64isplayName\x12\x19\n\x08is_local\x18\x03 \x01(\x08R\x07isLocal2\x8b\x04\n\x16PartyManagementService\x12\x81\x01\n\x10GetParticipantId\x12\x35.com.daml.ledger.api.v1.admin.GetParticipantIdRequest\x1a\x36.com.daml.ledger.api.v1.admin.GetParticipantIdResponse\x12o\n\nGetParties\x12/.com.daml.ledger.api.v1.admin.GetPartiesRequest\x1a\x30.com.daml.ledger.api.v1.admin.GetPartiesResponse\x12\x81\x01\n\x10ListKnownParties\x12\x35.com.daml.ledger.api.v1.admin.ListKnownPartiesRequest\x1a\x36.com.daml.ledger.api.v1.admin.ListKnownPartiesResponse\x12x\n\rAllocateParty\x12\x32.com.daml.ledger.api.v1.admin.AllocatePartyRequest\x1a\x33.com.daml.ledger.api.v1.admin.AllocatePartyResponseB\xac\x01\n\x1c\x63om.daml.ledger.api.v1.adminB PartyManagementServiceOuterClassZKgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1/admin\xaa\x02\x1c\x43om.Daml.Ledger.Api.V1.Adminb\x06proto3')



_GETPARTICIPANTIDREQUEST = DESCRIPTOR.message_types_by_name['GetParticipantIdRequest']
_GETPARTICIPANTIDRESPONSE = DESCRIPTOR.message_types_by_name['GetParticipantIdResponse']
_GETPARTIESREQUEST = DESCRIPTOR.message_types_by_name['GetPartiesRequest']
_GETPARTIESRESPONSE = DESCRIPTOR.message_types_by_name['GetPartiesResponse']
_LISTKNOWNPARTIESREQUEST = DESCRIPTOR.message_types_by_name['ListKnownPartiesRequest']
_LISTKNOWNPARTIESRESPONSE = DESCRIPTOR.message_types_by_name['ListKnownPartiesResponse']
_ALLOCATEPARTYREQUEST = DESCRIPTOR.message_types_by_name['AllocatePartyRequest']
_ALLOCATEPARTYRESPONSE = DESCRIPTOR.message_types_by_name['AllocatePartyResponse']
_PARTYDETAILS = DESCRIPTOR.message_types_by_name['PartyDetails']
GetParticipantIdRequest = _reflection.GeneratedProtocolMessageType('GetParticipantIdRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETPARTICIPANTIDREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.admin.party_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.GetParticipantIdRequest)
  })
_sym_db.RegisterMessage(GetParticipantIdRequest)

GetParticipantIdResponse = _reflection.GeneratedProtocolMessageType('GetParticipantIdResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETPARTICIPANTIDRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.admin.party_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.GetParticipantIdResponse)
  })
_sym_db.RegisterMessage(GetParticipantIdResponse)

GetPartiesRequest = _reflection.GeneratedProtocolMessageType('GetPartiesRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETPARTIESREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.admin.party_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.GetPartiesRequest)
  })
_sym_db.RegisterMessage(GetPartiesRequest)

GetPartiesResponse = _reflection.GeneratedProtocolMessageType('GetPartiesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETPARTIESRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.admin.party_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.GetPartiesResponse)
  })
_sym_db.RegisterMessage(GetPartiesResponse)

ListKnownPartiesRequest = _reflection.GeneratedProtocolMessageType('ListKnownPartiesRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTKNOWNPARTIESREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.admin.party_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.ListKnownPartiesRequest)
  })
_sym_db.RegisterMessage(ListKnownPartiesRequest)

ListKnownPartiesResponse = _reflection.GeneratedProtocolMessageType('ListKnownPartiesResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTKNOWNPARTIESRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.admin.party_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.ListKnownPartiesResponse)
  })
_sym_db.RegisterMessage(ListKnownPartiesResponse)

AllocatePartyRequest = _reflection.GeneratedProtocolMessageType('AllocatePartyRequest', (_message.Message,), {
  'DESCRIPTOR' : _ALLOCATEPARTYREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.admin.party_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.AllocatePartyRequest)
  })
_sym_db.RegisterMessage(AllocatePartyRequest)

AllocatePartyResponse = _reflection.GeneratedProtocolMessageType('AllocatePartyResponse', (_message.Message,), {
  'DESCRIPTOR' : _ALLOCATEPARTYRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.admin.party_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.AllocatePartyResponse)
  })
_sym_db.RegisterMessage(AllocatePartyResponse)

PartyDetails = _reflection.GeneratedProtocolMessageType('PartyDetails', (_message.Message,), {
  'DESCRIPTOR' : _PARTYDETAILS,
  '__module__' : 'com.daml.ledger.api.v1.admin.party_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.PartyDetails)
  })
_sym_db.RegisterMessage(PartyDetails)

_PARTYMANAGEMENTSERVICE = DESCRIPTOR.services_by_name['PartyManagementService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\034com.daml.ledger.api.v1.adminB PartyManagementServiceOuterClassZKgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1/admin\252\002\034Com.Daml.Ledger.Api.V1.Admin'
  _GETPARTICIPANTIDREQUEST._serialized_start=93
  _GETPARTICIPANTIDREQUEST._serialized_end=118
  _GETPARTICIPANTIDRESPONSE._serialized_start=120
  _GETPARTICIPANTIDRESPONSE._serialized_end=185
  _GETPARTIESREQUEST._serialized_start=187
  _GETPARTIESREQUEST._serialized_end=232
  _GETPARTIESRESPONSE._serialized_start=234
  _GETPARTIESRESPONSE._serialized_end=335
  _LISTKNOWNPARTIESREQUEST._serialized_start=337
  _LISTKNOWNPARTIESREQUEST._serialized_end=362
  _LISTKNOWNPARTIESRESPONSE._serialized_start=364
  _LISTKNOWNPARTIESRESPONSE._serialized_end=471
  _ALLOCATEPARTYREQUEST._serialized_start=473
  _ALLOCATEPARTYREQUEST._serialized_end=566
  _ALLOCATEPARTYRESPONSE._serialized_start=568
  _ALLOCATEPARTYRESPONSE._serialized_end=672
  _PARTYDETAILS._serialized_start=674
  _PARTYDETAILS._serialized_end=772
  _PARTYMANAGEMENTSERVICE._serialized_start=775
  _PARTYMANAGEMENTSERVICE._serialized_end=1298
# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
"""
Utilities for working with ledger offsets.

This API is _not_ public!
"""

from typing import Any, Optional, Union

__all__ = [
    "LedgerOffsetRange",
    "End",
    "END",
    "UNTIL_END",
    "FROM_BEGINNING_UNTIL_FOREVER",
    "from_offset_until_forever",
]


class End:
    """
    Marker object that denotes the current end of the ledger.
    """

    __slots__ = ()

    def __hash__(self):
        return 0

    def __repr__(self):
        return "END"


END = End()


class LedgerOffsetRange:
    """
    Denotes an offset range on a ledger. This API is _not_ considered public.

    The gRPC Ledger API and HTTP JSON API do not expose ledger offsets with the same semantics,
    so this class actually represents the commonality between these two interfaces.
    """

    def __init__(self, __begin: "Union[None, str]", __end: "Union[None, str, End]"):
        """
        Initialize a :class:`LedgerOffsetRange`.

        :param __begin:
            The start of the stream. If ``None``, then read from the beginning of the ledger.
            Otherwise, must be a legal ledger offset.
        :param __end:
            The end of the stream. If ``None``, then keep reading from the stream forever; if
            ``END``, then terminate when reaching the _current_ end of stream.

            Note that offsets are only allowed here on the gRPC Ledger API; they are *not*
            allowed here on the HTTP JSON API does not provide a mechanism for reading *to* a
            specific transaction offset.
        """
        self.begin = __begin
        self.end = __end

    def __eq__(self, other: Any) -> bool:
        return (
            isinstance(other, LedgerOffsetRange)
            and self.begin == other.begin
            and self.end == other.end
        )

    def __hash__(self):
        return hash(self.begin) ^ hash(self.end)

    def __repr__(self):
        return f"({self.begin}, {self.end})"


UNTIL_END = LedgerOffsetRange(None, END)
FROM_BEGINNING_UNTIL_FOREVER = LedgerOffsetRange(None, None)


def from_offset_until_forever(offset: Optional[str]) -> LedgerOffsetRange:
    return LedgerOffsetRange(offset, None)



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/admin/user_management_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n:com/daml/ledger/api/v1/admin/user_management_service.proto\x12\x1c\x63om.daml.ledger.api.v1.admin\";\n\x04User\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12#\n\rprimary_party\x18\x02 \x01(\tR\x0cprimaryParty\"\xec\x02\n\x05Right\x12\x63\n\x11participant_admin\x18\x01 \x01(\x0b\x32\x34.com.daml.ledger.api.v1.admin.Right.ParticipantAdminH\x00R\x10participantAdmin\x12L\n\ncan_act_as\x18\x02 \x01(\x0b\x32,.com.daml.ledger.api.v1.admin.Right.CanActAsH\x00R\x08\x63\x61nActAs\x12O\n\x0b\x63\x61n_read_as\x18\x03 \x01(\x0b\x32-.com.daml.ledger.api.v1.admin.Right.CanReadAsH\x00R\tcanReadAs\x1a\x12\n\x10ParticipantAdmin\x1a \n\x08\x43\x61nActAs\x12\x14\n\x05party\x18\x01 \x01(\tR\x05party\x1a!\n\tCanReadAs\x12\x14\n\x05party\x18\x01 \x01(\tR\x05partyB\x06\n\x04kind\"\x88\x01\n\x11\x43reateUserRequest\x12\x36\n\x04user\x18\x01 \x01(\x0b\x32\".com.daml.ledger.api.v1.admin.UserR\x04user\x12;\n\x06rights\x18\x02 \x03(\x0b\x32#.com.daml.ledger.api.v1.admin.RightR\x06rights\"L\n\x12\x43reateUserResponse\x12\x36\n\x04user\x18\x01 \x01(\x0b\x32\".com.daml.ledger.api.v1.admin.UserR\x04user\")\n\x0eGetUserRequest\x12\x17\n\x07user_id\x18\x01 \x01(\tR\x06userId\"I\n\x0fGetUserResponse\x12\x36\n\x04user\x18\x01 \x01(\x0b\x32\".com.daml.ledger.api.v1.admin.UserR\x04user\",\n\x11\x44\x65leteUserRequest\x12\x17\n\x07user_id\x18\x01 \x01(\tR\x06userId\"\x14\n\x12\x44\x65leteUserResponse\"N\n\x10ListUsersRequest\x12\x1d\n\npage_token\x18\x02 \x01(\tR\tpageToken\x12\x1b\n\tpage_size\x18\x03 \x01(\x05R\x08pageSize\"u\n\x11ListUsersResponse\x12\x38\n\x05users\x18\x01 \x03(\x0b\x32\".com.daml.ledger.api.v1.admin.UserR\x05users\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken\"n\n\x16GrantUserRightsRequest\x12\x17\n\x07user_id\x18\x01 \x01(\tR\x06userId\x12;\n\x06rights\x18\x02 \x03(\x0b\x32#.com.daml.ledger.api.v1.admin.RightR\x06rights\"p\n\x17GrantUserRightsResponse\x12U\n\x14newly_granted_rights\x18\x01 \x03(\x0b\x32#.com.daml.ledger.api.v1.admin.RightR\x12newlyGrantedRights\"o\n\x17RevokeUserRightsRequest\x12\x17\n\x07user_id\x18\x01 \x01(\tR\x06userId\x12;\n\x06rights\x18\x02 \x03(\x0b\x32#.com.daml.ledger.api.v1.admin.RightR\x06rights\"q\n\x18RevokeUserRightsResponse\x12U\n\x14newly_revoked_rights\x18\x01 \x03(\x0b\x32#.com.daml.ledger.api.v1.admin.RightR\x12newlyRevokedRights\"0\n\x15ListUserRightsRequest\x12\x17\n\x07user_id\x18\x01 \x01(\tR\x06userId\"U\n\x16ListUserRightsResponse\x12;\n\x06rights\x18\x01 \x03(\x0b\x32#.com.daml.ledger.api.v1.admin.RightR\x06rights2\xd0\x06\n\x15UserManagementService\x12o\n\nCreateUser\x12/.com.daml.ledger.api.v1.admin.CreateUserRequest\x1a\x30.com.daml.ledger.api.v1.admin.CreateUserResponse\x12\x66\n\x07GetUser\x12,.com.daml.ledger.api.v1.admin.GetUserRequest\x1a-.com.daml.ledger.api.v1.admin.GetUserResponse\x12o\n\nDeleteUser\x12/.com.daml.ledger.api.v1.admin.DeleteUserRequest\x1a\x30.com.daml.ledger.api.v1.admin.DeleteUserResponse\x12l\n\tListUsers\x12..com.daml.ledger.api.v1.admin.ListUsersRequest\x1a/.com.daml.ledger.api.v1.admin.ListUsersResponse\x12~\n\x0fGrantUserRights\x12\x34.com.daml.ledger.api.v1.admin.GrantUserRightsRequest\x1a\x35.com.daml.ledger.api.v1.admin.GrantUserRightsResponse\x12\x81\x01\n\x10RevokeUserRights\x12\x35.com.daml.ledger.api.v1.admin.RevokeUserRightsRequest\x1a\x36.com.daml.ledger.api.v1.admin.RevokeUserRightsResponse\x12{\n\x0eListUserRights\x12\x33.com.daml.ledger.api.v1.admin.ListUserRightsRequest\x1a\x34.com.daml.ledger.api.v1.admin.ListUserRightsResponseB\xab\x01\n\x1c\x63om.daml.ledger.api.v1.adminB\x1fUserManagementServiceOuterClassZKgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1/admin\xaa\x02\x1c\x43om.Daml.Ledger.Api.V1.Adminb\x06proto3')



_USER = DESCRIPTOR.message_types_by_name['User']
_RIGHT = DESCRIPTOR.message_types_by_name['Right']
_RIGHT_PARTICIPANTADMIN = _RIGHT.nested_types_by_name['ParticipantAdmin']
_RIGHT_CANACTAS = _RIGHT.nested_types_by_name['CanActAs']
_RIGHT_CANREADAS = _RIGHT.nested_types_by_name['CanReadAs']
_CREATEUSERREQUEST = DESCRIPTOR.message_types_by_name['CreateUserRequest']
_CREATEUSERRESPONSE = DESCRIPTOR.message_types_by_name['CreateUserResponse']
_GETUSERREQUEST = DESCRIPTOR.message_types_by_name['GetUserRequest']
_GETUSERRESPONSE = DESCRIPTOR.message_types_by_name['GetUserResponse']
_DELETEUSERREQUEST = DESCRIPTOR.message_types_by_name['DeleteUserRequest']
_DELETEUSERRESPONSE = DESCRIPTOR.message_types_by_name['DeleteUserResponse']
_LISTUSERSREQUEST = DESCRIPTOR.message_types_by_name['ListUsersRequest']
_LISTUSERSRESPONSE = DESCRIPTOR.message_types_by_name['ListUsersResponse']
_GRANTUSERRIGHTSREQUEST = DESCRIPTOR.message_types_by_name['GrantUserRightsRequest']
_GRANTUSERRIGHTSRESPONSE = DESCRIPTOR.message_types_by_name['GrantUserRightsResponse']
_REVOKEUSERRIGHTSREQUEST = DESCRIPTOR.message_types_by_name['RevokeUserRightsRequest']
_REVOKEUSERRIGHTSRESPONSE = DESCRIPTOR.message_types_by_name['RevokeUserRightsResponse']
_LISTUSERRIGHTSREQUEST = DESCRIPTOR.message_types_by_name['ListUserRightsRequest']
_LISTUSERRIGHTSRESPONSE = DESCRIPTOR.message_types_by_name['ListUserRightsResponse']
User = _reflection.GeneratedProtocolMessageType('User', (_message.Message,), {
  'DESCRIPTOR' : _USER,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.User)
  })
_sym_db.RegisterMessage(User)

Right = _reflection.GeneratedProtocolMessageType('Right', (_message.Message,), {

  'ParticipantAdmin' : _reflection.GeneratedProtocolMessageType('ParticipantAdmin', (_message.Message,), {
    'DESCRIPTOR' : _RIGHT_PARTICIPANTADMIN,
    '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
    # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.Right.ParticipantAdmin)
    })
  ,

  'CanActAs' : _reflection.GeneratedProtocolMessageType('CanActAs', (_message.Message,), {
    'DESCRIPTOR' : _RIGHT_CANACTAS,
    '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
    # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.Right.CanActAs)
    })
  ,

  'CanReadAs' : _reflection.GeneratedProtocolMessageType('CanReadAs', (_message.Message,), {
    'DESCRIPTOR' : _RIGHT_CANREADAS,
    '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
    # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.Right.CanReadAs)
    })
  ,
  'DESCRIPTOR' : _RIGHT,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.Right)
  })
_sym_db.RegisterMessage(Right)
_sym_db.RegisterMessage(Right.ParticipantAdmin)
_sym_db.RegisterMessage(Right.CanActAs)
_sym_db.RegisterMessage(Right.CanReadAs)

CreateUserRequest = _reflection.GeneratedProtocolMessageType('CreateUserRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEUSERREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.CreateUserRequest)
  })
_sym_db.RegisterMessage(CreateUserRequest)

CreateUserResponse = _reflection.GeneratedProtocolMessageType('CreateUserResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATEUSERRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.CreateUserResponse)
  })
_sym_db.RegisterMessage(CreateUserResponse)

GetUserRequest = _reflection.GeneratedProtocolMessageType('GetUserRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETUSERREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.GetUserRequest)
  })
_sym_db.RegisterMessage(GetUserRequest)

GetUserResponse = _reflection.GeneratedProtocolMessageType('GetUserResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETUSERRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.GetUserResponse)
  })
_sym_db.RegisterMessage(GetUserResponse)

DeleteUserRequest = _reflection.GeneratedProtocolMessageType('DeleteUserRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEUSERREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.DeleteUserRequest)
  })
_sym_db.RegisterMessage(DeleteUserRequest)

DeleteUserResponse = _reflection.GeneratedProtocolMessageType('DeleteUserResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETEUSERRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.DeleteUserResponse)
  })
_sym_db.RegisterMessage(DeleteUserResponse)

ListUsersRequest = _reflection.GeneratedProtocolMessageType('ListUsersRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTUSERSREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.ListUsersRequest)
  })
_sym_db.RegisterMessage(ListUsersRequest)

ListUsersResponse = _reflection.GeneratedProtocolMessageType('ListUsersResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTUSERSRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.ListUsersResponse)
  })
_sym_db.RegisterMessage(ListUsersResponse)

GrantUserRightsRequest = _reflection.GeneratedProtocolMessageType('GrantUserRightsRequest', (_message.Message,), {
  'DESCRIPTOR' : _GRANTUSERRIGHTSREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.GrantUserRightsRequest)
  })
_sym_db.RegisterMessage(GrantUserRightsRequest)

GrantUserRightsResponse = _reflection.GeneratedProtocolMessageType('GrantUserRightsResponse', (_message.Message,), {
  'DESCRIPTOR' : _GRANTUSERRIGHTSRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.GrantUserRightsResponse)
  })
_sym_db.RegisterMessage(GrantUserRightsResponse)

RevokeUserRightsRequest = _reflection.GeneratedProtocolMessageType('RevokeUserRightsRequest', (_message.Message,), {
  'DESCRIPTOR' : _REVOKEUSERRIGHTSREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.RevokeUserRightsRequest)
  })
_sym_db.RegisterMessage(RevokeUserRightsRequest)

RevokeUserRightsResponse = _reflection.GeneratedProtocolMessageType('RevokeUserRightsResponse', (_message.Message,), {
  'DESCRIPTOR' : _REVOKEUSERRIGHTSRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.RevokeUserRightsResponse)
  })
_sym_db.RegisterMessage(RevokeUserRightsResponse)

ListUserRightsRequest = _reflection.GeneratedProtocolMessageType('ListUserRightsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTUSERRIGHTSREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.ListUserRightsRequest)
  })
_sym_db.RegisterMessage(ListUserRightsRequest)

ListUserRightsResponse = _reflection.GeneratedProtocolMessageType('ListUserRightsResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTUSERRIGHTSRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.ListUserRightsResponse)
  })
_sym_db.RegisterMessage(ListUserRightsResponse)

_USERMANAGEMENTSERVICE = DESCRIPTOR.services_by_name['UserManagementService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\034com.daml.ledger.api.v1.adminB\037UserManagementServiceOuterClassZKgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1/admin\252\002\034Com.Daml.Ledger.Api.V1.Admin'
  _USER._serialized_start=92
  _USER._serialized_end=151
  _RIGHT._serialized_start=154
  _RIGHT._serialized_end=518
  _RIGHT_PARTICIPANTADMIN._serialized_start=423
  _RIGHT_PARTICIPANTADMIN._serialized_end=441
  _RIGHT_CANACTAS._serialized_start=443
  _RIGHT_CANACTAS._serialized_end=475
  _RIGHT_CANREADAS._serialized_start=477
  _RIGHT_CANREADAS._serialized_end=510
  _CREATEUSERREQUEST._serialized_start=521
  _CREATEUSERREQUEST._serialized_end=657
  _CREATEUSERRESPONSE._serialized_start=659
  _CREATEUSERRESPONSE._serialized_end=735
  _GETUSERREQUEST._serialized_start=737
  _GETUSERREQUEST._serialized_end=778
  _GETUSERRESPONSE._serialized_start=780
  _GETUSERRESPONSE._serialized_end=853
  _DELETEUSERREQUEST._serialized_start=855
  _DELETEUSERREQUEST._serialized_end=899
  _DELETEUSERRESPONSE._serialized_start=901
  _DELETEUSERRESPONSE._serialized_end=921
  _LISTUSERSREQUEST._serialized_start=923
  _LISTUSERSREQUEST._serialized_end=1001
  _LISTUSERSRESPONSE._serialized_start=1003
  _LISTUSERSRESPONSE._serialized_end=1120
  _GRANTUSERRIGHTSREQUEST._serialized_start=1122
  _GRANTUSERRIGHTSREQUEST._serialized_end=1232
  _GRANTUSERRIGHTSRESPONSE._serialized_start=1234
  _GRANTUSERRIGHTSRESPONSE._serialized_end=1346
  _REVOKEUSERRIGHTSREQUEST._serialized_start=1348
  _REVOKEUSERRIGHTSREQUEST._serialized_end=1459
  _REVOKEUSERRIGHTSRESPONSE._serialized_start=1461
  _REVOKEUSERRIGHTSRESPONSE._serialized_end=1574
  _LISTUSERRIGHTSREQUEST._serialized_start=1576
  _LISTUSERRIGHTSREQUEST._serialized_end=1624
  _LISTUSERRIGHTSRESPONSE._serialized_start=1626
  _LISTUSERRIGHTSRESPONSE._serialized_end=1711
  _USERMANAGEMENTSERVICE._serialized_start=1714
  _USERMANAGEMENTSERVICE._serialized_end=2562
# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
Error types
-----------

.. autoclass:: PackageNotFoundError
.. autoclass:: NameNotFoundError
"""

from typing import TYPE_CHECKING, Any

from ..prim import DazlError

if TYPE_CHECKING:
    from . import daml_lf_1

__all__ = ["PackageNotFoundError", "NameNotFoundError"]


class PackageNotFoundError(DazlError):
    """
    Raised when a :class:`Package` was expected but could not be found.

    This error can also be raised in places where types/values are being resolved; it is an
    indication that the operation MAY succeed if the package of the specified ID is fetched and
    the original option retried.
    """

    def __init__(self, ref: "daml_lf_1.PackageRef"):
        super().__init__(ref)
        self.ref = ref


class NameNotFoundError(DazlError):
    """
    Raised when a :class:`daml_lf_1.DefDataType`, :class:`daml_lf_1.DefValue`, or
    :class:`daml_lf_1.DefTemplate` of a specific name was expected, but could not be found.

    Typically this error is raised when the *package ID* is valid, but the name is not. Because
    package IDs are immutable, this error is not normally retryable.
    """

    def __init__(self, ref: "Any"):
        super().__init__(ref)
        self.ref = ref



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file

import builtins as _builtins, sys, typing as _typing

from google.protobuf.internal.containers import RepeatedCompositeFieldContainer
from google.protobuf.message import Message as _Message
from google.protobuf.timestamp_pb2 import Timestamp

if sys.version_info >= (3, 8):
    from typing import Literal as _L
else:
    from typing_extensions import Literal as _L

__all__ = [
    "ListKnownPackagesRequest",
    "ListKnownPackagesResponse",
    "PackageDetails",
    "UploadDarFileRequest",
    "UploadDarFileResponse",
]


class ListKnownPackagesRequest(_Message):
    def __init__(self): ...
    def HasField(self, field_name: _typing.NoReturn) -> _typing.NoReturn: ...
    def ClearField(self, field_name: _typing.NoReturn) -> _typing.NoReturn: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class ListKnownPackagesResponse(_Message):
    @property
    def package_details(self) -> RepeatedCompositeFieldContainer[PackageDetails]: ...
    def __init__(self, *, package_details: _typing.Optional[_typing.Iterable[PackageDetails]] = ...): ...
    def HasField(self, field_name: _L["package_details"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["package_details"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class PackageDetails(_Message):
    package_id: _builtins.str
    package_size: _builtins.int
    @property
    def known_since(self) -> Timestamp: ...
    source_description: _builtins.str
    def __init__(self, *, package_id: _typing.Optional[_builtins.str] = ..., package_size: _typing.Optional[_builtins.int] = ..., known_since: _typing.Optional[Timestamp] = ..., source_description: _typing.Optional[_builtins.str] = ...): ...
    def HasField(self, field_name: _L["package_id", "package_size", "known_since", "source_description"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["package_id", "package_size", "known_since", "source_description"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class UploadDarFileRequest(_Message):
    dar_file: _builtins.bytes
    submission_id: _builtins.str
    def __init__(self, *, dar_file: _typing.Optional[_builtins.bytes] = ..., submission_id: _typing.Optional[_builtins.str] = ...): ...
    def HasField(self, field_name: _L["dar_file", "submission_id"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _L["dar_file", "submission_id"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class UploadDarFileResponse(_Message):
    def __init__(self): ...
    def HasField(self, field_name: _typing.NoReturn) -> _typing.NoReturn: ...
    def ClearField(self, field_name: _typing.NoReturn) -> _typing.NoReturn: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...



# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import metering_report_service_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_metering__report__service__pb2


class MeteringReportServiceStub(object):
    """Experimental API to retrieve metering reports.

    Metering reports aim to provide the information necessary for billing participant
    and application operators.

    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetMeteringReport = channel.unary_unary(
                '/com.daml.ledger.api.v1.admin.MeteringReportService/GetMeteringReport',
                request_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_metering__report__service__pb2.GetMeteringReportRequest.SerializeToString,
                response_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_metering__report__service__pb2.GetMeteringReportResponse.FromString,
                )


class MeteringReportServiceServicer(object):
    """Experimental API to retrieve metering reports.

    Metering reports aim to provide the information necessary for billing participant
    and application operators.

    """

    def GetMeteringReport(self, request, context):
        """Retrieve a metering report.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_MeteringReportServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetMeteringReport': grpc.unary_unary_rpc_method_handler(
                    servicer.GetMeteringReport,
                    request_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_metering__report__service__pb2.GetMeteringReportRequest.FromString,
                    response_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_metering__report__service__pb2.GetMeteringReportResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'com.daml.ledger.api.v1.admin.MeteringReportService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class MeteringReportService(object):
    """Experimental API to retrieve metering reports.

    Metering reports aim to provide the information necessary for billing participant
    and application operators.

    """

    @staticmethod
    def GetMeteringReport(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/com.daml.ledger.api.v1.admin.MeteringReportService/GetMeteringReport',
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_metering__report__service__pb2.GetMeteringReportRequest.SerializeToString,
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_metering__report__service__pb2.GetMeteringReportResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)
